@@ -2,7 +2,7 @@
 import logging
 import os
 from collections import defaultdict
-from typing import Dict, List, Optional
+from typing import Dict, List, Optional, Tuple
 
 import sqlalchemy as sa
 from sqlalchemy.dialects import postgresql  # noqa
@@ -26,7 +26,12 @@ from .exc import (
     TableNotFoundError,
 )
 from .node import Node
-from .settings import PG_SSLMODE, PG_SSLROOTCERT, QUERY_CHUNK_SIZE
+from .settings import (
+    PG_SSLMODE,
+    PG_SSLROOTCERT,
+    QUERY_CHUNK_SIZE,
+    STREAM_RESULTS,
+)
 from .trigger import CREATE_TRIGGER_TEMPLATE
 from .urls import get_postgres_url
 from .utils import compiled_query
@@ -68,10 +73,12 @@ class TupleIdentifierType(sa.types.UserDefinedType):
 class Base(object):
     def __init__(self, database: str, verbose: bool = False, *args, **kwargs):
         """Initialize the base class constructor."""
-        self.__engine: sa.engine.Engine = pg_engine(database, **kwargs)
+        self.__engine: sa.engine.Engine = _pg_engine(
+            database, echo=False, **kwargs
+        )
         self.__schemas: Optional[dict] = None
         # models is a dict of f'{schema}.{table}'
-        self.models: Dict[str] = {}
+        self.__models: Dict[str] = {}
         self.__metadata: Dict[str] = {}
         self.__indices: Dict[str] = {}
         self.__views: Dict[str] = {}
@@ -83,7 +90,7 @@ class Base(object):
     def connect(self) -> None:
         """Connect to database."""
         try:
-            conn = self.__engine.connect()
+            conn = self.engine.connect()
             conn.close()
         except Exception as e:
             logger.exception(f"Cannot connect to database: {e}")
@@ -117,7 +124,7 @@ class Base(object):
         host_part: str = self.engine.url.host.split(".")[0]
         username: str = username.split(f"@{host_part}")[0]
 
-        with self.__engine.connect() as conn:
+        with self.engine.connect() as conn:
             return (
                 conn.execute(
                     sa.select([sa.column("usename")])
@@ -145,7 +152,7 @@ class Base(object):
             )
 
     # Tables...
-    def model(self, table: str, schema: str) -> sa.sql.Alias:
+    def models(self, table: str, schema: str) -> sa.sql.Alias:
         """Get an SQLAlchemy model representation from a table.
 
         Args:
@@ -157,10 +164,10 @@ class Base(object):
 
         """
         name: str = f"{schema}.{table}"
-        if name not in self.models:
+        if name not in self.__models:
             if schema not in self.__metadata:
                 metadata = sa.MetaData(schema=schema)
-                metadata.reflect(self.__engine, views=True)
+                metadata.reflect(self.engine, views=True)
                 self.__metadata[schema] = metadata
             metadata = self.__metadata[schema]
             if name not in metadata.tables:
@@ -182,9 +189,9 @@ class Base(object):
                 "primary_keys",
                 sorted([primary_key.key for primary_key in model.primary_key]),
             )
-            self.models[f"{model.original}"] = model
+            self.__models[f"{model.original}"] = model
 
-        return self.models[name]
+        return self.__models[name]
 
     @property
     def conn(self):
@@ -193,11 +200,11 @@ class Base(object):
     @property
     def database(self) -> str:
         """str: Get the database name."""
-        return self.__engine.url.database
+        return self.engine.url.database
 
     @property
     def session(self) -> sessionmaker:
-        Session = sessionmaker(bind=self.__engine.connect(), autoflush=True)
+        Session = sessionmaker(bind=self.engine.connect(), autoflush=True)
         return Session()
 
     @property
@@ -240,16 +247,16 @@ class Base(object):
     def indices(self, table: str, schema: str) -> list:
         """Get the database table indexes."""
         if table not in self.__indices:
-            self.__indices[table] = sa.inspect(self.engine).get_indexes(
-                table, schema=schema
+            self.__indices[table] = sorted(
+                sa.inspect(self.engine).get_indexes(table, schema=schema)
             )
         return self.__indices[table]
 
     def tables(self, schema: str) -> list:
         """Get the table names for current schema."""
         if schema not in self.__tables:
-            self.__tables[schema] = sa.inspect(self.engine).get_table_names(
-                schema
+            self.__tables[schema] = sorted(
+                sa.inspect(self.engine).get_table_names(schema)
             )
         return self.__tables[schema]
 
@@ -266,8 +273,7 @@ class Base(object):
 
         """
         logger.debug(f"Truncating table: {schema}.{table}")
-        query = f'TRUNCATE TABLE "{schema}"."{table}" CASCADE'
-        self.execute(query)
+        self.execute(sa.DDL(f'TRUNCATE TABLE "{schema}"."{table}" CASCADE'))
 
     def truncate_tables(
         self, tables: List[str], schema: str = DEFAULT_SCHEMA
@@ -402,7 +408,7 @@ class Base(object):
         upto_nchanges: Optional[int] = None,
         limit: Optional[int] = None,
         offset: Optional[int] = None,
-    ) -> List[sa.engine.row.LegacyRow]:
+    ) -> None:
         """Get/Consume changes from a logical replication slot.
 
         To get one change and data in existing replication slot:
@@ -421,7 +427,7 @@ class Base(object):
             limit=limit,
             offset=offset,
         )
-        return self.fetchall(statement)
+        self.execute(statement)
 
     def logical_slot_peek_changes(
         self,
@@ -465,7 +471,7 @@ class Base(object):
             upto_lsn=upto_lsn,
             upto_nchanges=upto_nchanges,
         )
-        with self.__engine.connect() as conn:
+        with self.engine.connect() as conn:
             return conn.execute(
                 statement.with_only_columns([sa.func.COUNT()])
             ).scalar()
@@ -476,7 +482,7 @@ class Base(object):
     ) -> None:
         create_view(
             self.engine,
-            self.model,
+            self.models,
             self.fetchall,
             schema,
             tables,
@@ -528,7 +534,7 @@ class Base(object):
                 )
         if join_queries:
             if queries:
-                self.execute("; ".join(queries))
+                self.execute(sa.DDL("; ".join(queries)))
         else:
             for query in queries:
                 self.execute(sa.DDL(query))
@@ -552,7 +558,7 @@ class Base(object):
                 )
         if join_queries:
             if queries:
-                self.execute("; ".join(queries))
+                self.execute(sa.DDL("; ".join(queries)))
         else:
             for query in queries:
                 self.execute(sa.DDL(query))
@@ -570,7 +576,9 @@ class Base(object):
 
     def drop_function(self, schema: str) -> None:
         self.execute(
-            f'DROP FUNCTION IF EXISTS "{schema}".{TRIGGER_FUNC}() CASCADE'
+            sa.DDL(
+                f'DROP FUNCTION IF EXISTS "{schema}".{TRIGGER_FUNC}() CASCADE'
+            )
         )
 
     def disable_triggers(self, schema: str) -> None:
@@ -578,22 +586,24 @@ class Base(object):
         for table in self.tables(schema):
             logger.debug(f"Disabling trigger on table: {schema}.{table}")
             for name in ("notify", "truncate"):
-                query = (
-                    f'ALTER TABLE "{schema}"."{table}" '
-                    f"DISABLE TRIGGER {table}_{name}"
+                self.execute(
+                    sa.DDL(
+                        f'ALTER TABLE "{schema}"."{table}" '
+                        f"DISABLE TRIGGER {table}_{name}"
+                    )
                 )
-                self.execute(query)
 
     def enable_triggers(self, schema: str) -> None:
         """Enable all pgsync defined triggers in database."""
         for table in self.tables(schema):
             logger.debug(f"Enabling trigger on table: {schema}.{table}")
             for name in ("notify", "truncate"):
-                query = (
-                    f'ALTER TABLE "{schema}"."{table}" '
-                    f"ENABLE TRIGGER {table}_{name}"
+                self.execute(
+                    sa.DDL(
+                        f'ALTER TABLE "{schema}"."{table}" '
+                        f"ENABLE TRIGGER {table}_{name}"
+                    )
                 )
-                self.execute(query)
 
     @property
     def txid_current(self) -> int:
@@ -645,10 +655,7 @@ class Base(object):
         ):
             value: str = value.lstrip("'").rstrip("'")
         if type_.lower() == "boolean":
-            try:
-                value: bool = bool(value)
-            except ValueError:
-                raise
+            value: bool = bool(value)
         if type_.lower() in (
             "double precision",
             "float4",
@@ -661,8 +668,8 @@ class Base(object):
                 raise
         return value
 
-    def parse_logical_slot(self, row: str):
-        def _parse_logical_slot(data):
+    def parse_logical_slot(self, row: str) -> dict:
+        def _parse_logical_slot(data: str) -> Tuple[str, str]:
 
             while True:
 
@@ -670,11 +677,11 @@ class Base(object):
                 if not match:
                     break
 
-                key = match.groupdict().get("key")
+                key: str = match.groupdict().get("key")
                 if key:
                     key = key.replace('"', "")
-                value = match.groupdict().get("value")
-                type_ = match.groupdict().get("type")
+                value: str = match.groupdict().get("value")
+                type_: str = match.groupdict().get("type")
 
                 value = self.parse_value(type_, value)
 
@@ -702,23 +709,24 @@ class Base(object):
                 msg = f"Unknown {tg_op} operation for row: {row}"
                 raise LogicalSlotParseError(msg)
 
-            i = suffix.index("old-key:")
+            i: int = suffix.index("old-key:")
             if i > -1:
-                j = suffix.index("new-tuple:")
-                s = suffix[i + len("old-key:") : j]
+                j: int = suffix.index("new-tuple:")
+                s: str = suffix[i + len("old-key:") : j]
                 for key, value in _parse_logical_slot(s):
                     payload["old"][key] = value
 
             i = suffix.index("new-tuple:")
             if i > -1:
-                s = suffix[i + len("new-tuple:") :]
+                s: str = suffix[i + len("new-tuple:") :]
                 for key, value in _parse_logical_slot(s):
                     payload["new"][key] = value
         else:
             # this can be an INSERT, DELETE, UPDATE or TRUNCATE operation
             if tg_op not in TG_OP:
-                message = f"Unknown {tg_op} operation for row: {row}"
-                raise LogicalSlotParseError(message)
+                raise LogicalSlotParseError(
+                    f"Unknown {tg_op} operation for row: {row}"
+                )
 
             for key, value in _parse_logical_slot(suffix):
                 payload["new"][key] = value
@@ -729,19 +737,11 @@ class Base(object):
     def execute(
         self,
         statement: sa.sql.Select,
-        values: dict = None,
-        options: dict = None,
+        values: Optional[dict] = None,
+        options: Optional[dict] = None,
     ) -> None:
         """Execute a query statement."""
-        conn = self.__engine.connect()
-        try:
-            if options:
-                conn = conn.execution_options(**options)
-            conn.execute(statement, values)
-            conn.close()
-        except Exception as e:
-            logger.exception(f"Exception {e}")
-            raise
+        pg_execute(self.engine, statement, values=values, options=options)
 
     def fetchone(
         self, statement: sa.sql.Select, label=None, literal_binds=False
@@ -750,7 +750,7 @@ class Base(object):
         if self.verbose:
             compiled_query(statement, label=label, literal_binds=literal_binds)
 
-        conn = self.__engine.connect()
+        conn = self.engine.connect()
         try:
             row = conn.execute(statement).fetchone()
             conn.close()
@@ -769,7 +769,7 @@ class Base(object):
         if self.verbose:
             compiled_query(statement, label=label, literal_binds=literal_binds)
 
-        conn = self.__engine.connect()
+        conn = self.engine.connect()
         try:
             rows = conn.execute(statement).fetchall()
             conn.close()
@@ -782,18 +782,20 @@ class Base(object):
         self,
         statement: sa.sql.Select,
         chunk_size: Optional[int] = None,
+        stream_results: Optional[bool] = None,
     ):
         chunk_size: int = chunk_size or QUERY_CHUNK_SIZE
-        with self.__engine.connect() as conn:
-            result = conn.execution_options(stream_results=True).execute(
-                statement.select()
-            )
+        stream_results: bool = stream_results or STREAM_RESULTS
+        with self.engine.connect() as conn:
+            result = conn.execution_options(
+                stream_results=stream_results
+            ).execute(statement.select())
             for partition in result.partitions(chunk_size):
                 for keys, row, *primary_keys in partition:
                     yield keys, row, primary_keys
 
     def fetchcount(self, statement: sa.sql.Subquery) -> int:
-        with self.__engine.connect() as conn:
+        with self.engine.connect() as conn:
             return conn.execute(
                 statement.original.with_only_columns(
                     [sa.func.COUNT()]
@@ -824,7 +826,7 @@ def subtransactions(session):
     return ControlledExecution(session)
 
 
-def _get_foreign_keys(model_a, model_b) -> dict:
+def _get_foreign_keys(model_a: sa.sql.Alias, model_b: sa.sql.Alias) -> dict:
 
     foreign_keys: dict = defaultdict(list)
 
@@ -902,6 +904,68 @@ def pg_engine(
     echo: bool = False,
     sslmode: Optional[str] = None,
     sslrootcert: Optional[str] = None,
+):
+    """Context manager for managing engines."""
+
+    class ControlledExecution:
+        def __init__(
+            self,
+            database: str,
+            user: Optional[str] = None,
+            host: Optional[str] = None,
+            password: Optional[str] = None,
+            port: Optional[str] = None,
+            echo: bool = False,
+            sslmode: Optional[str] = None,
+            sslrootcert: Optional[str] = None,
+        ):
+            self.database = database
+            self.user = user
+            self.host = host
+            self.password = password
+            self.port = port
+            self.echo = echo
+            self.sslmode = sslmode
+            self.sslrootcert = sslrootcert
+
+        def __enter__(self) -> sa.engine.Engine:
+            self._engine = _pg_engine(
+                database,
+                user=self.user,
+                host=self.host,
+                password=self.password,
+                port=self.port,
+                echo=self.echo,
+                sslmode=self.sslmode,
+                sslrootcert=self.sslrootcert,
+            )
+            return self._engine
+
+        def __exit__(self, type, value, traceback) -> None:
+            self._engine.connect().close()
+            self._engine.dispose()
+
+    return ControlledExecution(
+        database,
+        user=user,
+        host=host,
+        password=host,
+        port=port,
+        echo=echo,
+        sslmode=sslmode,
+        sslrootcert=sslrootcert,
+    )
+
+
+def _pg_engine(
+    database: str,
+    user: Optional[str] = None,
+    host: Optional[str] = None,
+    password: Optional[str] = None,
+    port: Optional[str] = None,
+    echo: bool = False,
+    sslmode: Optional[str] = None,
+    sslrootcert: Optional[str] = None,
 ) -> sa.engine.Engine:
     connect_args: dict = {}
     sslmode: str = sslmode or PG_SSLMODE
@@ -940,7 +1004,7 @@ def pg_engine(
 
 def pg_execute(
     engine: sa.engine.Engine,
-    query: str,
+    statement: sa.sql.Select,
     values: Optional[list] = None,
     options: Optional[dict] = None,
 ) -> None:
@@ -949,32 +1013,34 @@ def pg_execute(
     try:
         if options:
             conn = conn.execution_options(**options)
-        conn.execute(query, values)
+        conn.execute(statement, values)
         conn.close()
     except Exception as e:
         logger.exception(f"Exception {e}")
         raise
 
 
-def create_schema(engine: sa.engine.Engine, schema: str) -> None:
+def create_schema(database: str, schema: str, echo: bool = False) -> None:
     """Create database schema."""
-    if schema != DEFAULT_SCHEMA:
-        engine.execute(sa.schema.CreateSchema(schema))
+    logger.debug(f"Creating schema: {schema}")
+    with pg_engine(database, echo=echo) as engine:
+        pg_execute(engine, sa.DDL(f"CREATE SCHEMA IF NOT EXISTS {schema}"))
+    logger.debug(f"Created schema: {schema}")
 
 
 def create_database(database: str, echo: bool = False) -> None:
     """Create a database."""
     logger.debug(f"Creating database: {database}")
-    engine: sa.engine.Engine = pg_engine(database="postgres", echo=echo)
-    pg_execute(engine, f'CREATE DATABASE "{database}"')
+    with pg_engine("postgres", echo=echo) as engine:
+        pg_execute(engine, sa.DDL(f'CREATE DATABASE "{database}"'))
     logger.debug(f"Created database: {database}")
 
 
 def drop_database(database: str, echo: bool = False) -> None:
     """Drop a database."""
     logger.debug(f"Dropping database: {database}")
-    engine: sa.engine.Engine = pg_engine(database="postgres", echo=echo)
-    pg_execute(engine, f'DROP DATABASE IF EXISTS "{database}"')
+    with pg_engine("postgres", echo=echo) as engine:
+        pg_execute(engine, sa.DDL(f'DROP DATABASE IF EXISTS "{database}"'))
     logger.debug(f"Dropped database: {database}")
 
 
@@ -983,14 +1049,17 @@ def create_extension(
 ) -> None:
     """Create a database extension."""
     logger.debug(f"Creating extension: {extension}")
-    engine: sa.engine.Engine = pg_engine(database=database, echo=echo)
-    pg_execute(engine, f'CREATE EXTENSION IF NOT EXISTS "{extension}"')
+    with pg_engine(database, echo=echo) as engine:
+        pg_execute(
+            engine,
+            sa.DDL(f'CREATE EXTENSION IF NOT EXISTS "{extension}"'),
+        )
     logger.debug(f"Created extension: {extension}")
 
 
 def drop_extension(database: str, extension: str, echo: bool = False) -> None:
     """Drop a database extension."""
     logger.debug(f"Dropping extension: {extension}")
-    engine: sa.engine.Engine = pg_engine(database=database, echo=echo)
-    pg_execute(engine, f'DROP EXTENSION IF EXISTS "{extension}"')
+    with pg_engine(database, echo=echo) as engine:
+        pg_execute(engine, sa.DDL(f'DROP EXTENSION IF EXISTS "{extension}"'))
     logger.debug(f"Dropped extension: {extension}")
