@@ -34,17 +34,15 @@ class Plugins(object):
 
     def walk(self, package: str) -> None:
         """Recursively walk the supplied package and fetch all plugins."""
-        plugins = import_module(package)
+        module = import_module(package)
         for _, name, ispkg in iter_modules(
-            plugins.__path__,
-            f"{plugins.__name__}.",
+            module.__path__,
+            prefix=f"{module.__name__}.",
         ):
             if ispkg:
                 continue
 
-            module = import_module(name)
-            members = getmembers(module, isclass)
-            for _, klass in members:
+            for _, klass in getmembers(import_module(name), isclass):
                 if issubclass(klass, Plugin) & (klass is not Plugin):
                     if klass.name not in self.names:
                         continue
@@ -54,10 +52,10 @@ class Plugins(object):
                     self.plugins.append(klass())
 
         paths: list = []
-        if isinstance(plugins.__path__, str):
-            paths.append(plugins.__path__)
+        if isinstance(module.__path__, str):
+            paths.append(module.__path__)
         else:
-            paths.extend([path for path in plugins.__path__])
+            paths.extend([path for path in module.__path__])
 
         for pkg_path in paths:
 
