@@ -95,10 +95,10 @@ def compile_refresh_view(
 
 
 class CreateIndex(DDLElement):
-    def __init__(self, name: str, schema: str, view: str, columns: list):
+    def __init__(self, name: str, schema: str, entity: str, columns: list):
         self.schema: str = schema
         self.name: str = name
-        self.view: str = view
+        self.entity: str = entity
         self.columns: list = columns
 
 
@@ -108,7 +108,7 @@ def compile_create_index(
 ) -> str:
     return (
         f"CREATE UNIQUE INDEX {element.name} ON "
-        f'"{element.schema}"."{element.view}" ({", ".join(element.columns)})'
+        f'"{element.schema}"."{element.entity}" ({", ".join(element.columns)})'
     )
 
 
@@ -125,7 +125,7 @@ def compile_drop_index(
 
 
 def _get_constraints(
-    model: Callable,
+    models: Callable,
     schema: str,
     tables: List[str],
     label: str,
@@ -133,11 +133,8 @@ def _get_constraints(
 ) -> sa.sql.Select:
     with warnings.catch_warnings():
         warnings.simplefilter("ignore", category=sa.exc.SAWarning)
-        table_constraints = model("table_constraints", "information_schema")
-        key_column_usage = model("key_column_usage", "information_schema")
-        constraint_column_usage = model(
-            "constraint_column_usage", "information_schema"
-        )
+        table_constraints = models("table_constraints", "information_schema")
+        key_column_usage = models("key_column_usage", "information_schema")
     return (
         sa.select(
             [
@@ -160,15 +157,6 @@ def _get_constraints(
                 key_column_usage.c.table_schema == schema,
             ),
         )
-        # .join(
-        #     constraint_column_usage,
-        #     sa.and_(
-        #         constraint_column_usage.c.constraint_name
-        #         == table_constraints.c.constraint_name,
-        #         constraint_column_usage.c.table_schema
-        #         == table_constraints.c.table_schema,
-        #     ),
-        # )
         .where(
             *[
                 table_constraints.c.table_name.in_(tables),
@@ -180,10 +168,10 @@ def _get_constraints(
 
 
 def _primary_keys(
-    model: Callable, schema: str, tables: List[str]
+    models: Callable, schema: str, tables: List[str]
 ) -> sa.sql.Select:
     return _get_constraints(
-        model,
+        models,
         schema,
         tables,
         label="primary_keys",
@@ -192,10 +180,10 @@ def _primary_keys(
 
 
 def _foreign_keys(
-    model: Callable, schema: str, tables: List[str]
+    models: Callable, schema: str, tables: List[str]
 ) -> sa.sql.Select:
     return _get_constraints(
-        model,
+        models,
         schema,
         tables,
         label="foreign_keys",
@@ -205,7 +193,7 @@ def _foreign_keys(
 
 def create_view(
     engine: sa.engine.Engine,
-    model: Callable,
+    models: Callable,
     fetchall: Callable,
     schema: str,
     tables: list,
@@ -255,7 +243,7 @@ def create_view(
         for table in set(tables):
             tables.add(f"{schema}.{table}")
 
-    for table_name, columns in fetchall(_primary_keys(model, schema, tables)):
+    for table_name, columns in fetchall(_primary_keys(models, schema, tables)):
         rows.setdefault(
             table_name,
             {"primary_keys": set(), "foreign_keys": set()},
@@ -263,7 +251,7 @@ def create_view(
         if columns:
             rows[table_name]["primary_keys"] |= set(columns)
 
-    for table_name, columns in fetchall(_foreign_keys(model, schema, tables)):
+    for table_name, columns in fetchall(_foreign_keys(models, schema, tables)):
         rows.setdefault(
             table_name,
             {"primary_keys": set(), "foreign_keys": set()},
