@@ -16,7 +16,7 @@ from pgsync.node import Tree
 from pgsync.settings import NTHREADS_POLLDB
 from pgsync.sync import Sync
 
-from .helpers.utils import assert_resync_empty, noop, search
+from .helpers.utils import assert_resync_empty, noop, search, sort_list
 
 
 @pytest.mark.usefixtures("table_creator")
@@ -112,8 +112,9 @@ class TestParentSingleChildFkOnChild(object):
                 }
             ],
         }
-        sync.nodes = nodes
-        docs = [doc for doc in sync.sync()]
+        sync.tree.__post_init__()
+        sync.root = sync.tree.build(nodes)
+        docs = [sort_list(doc) for doc in sync.sync()]
         assert docs[0]["_id"] == "abc"
         assert docs[0]["_source"] == {
             "_meta": {"rating": {"id": [1]}},
@@ -157,8 +158,9 @@ class TestParentSingleChildFkOnChild(object):
                 }
             ],
         }
-        sync.nodes = nodes
-        docs = [doc for doc in sync.sync()]
+        sync.tree.__post_init__()
+        sync.root = sync.tree.build(nodes)
+        docs = [sort_list(doc) for doc in sync.sync()]
         docs = sorted(docs, key=lambda k: k["_id"])
 
         assert docs[0]["_id"] == "abc"
@@ -204,8 +206,9 @@ class TestParentSingleChildFkOnChild(object):
                 }
             ],
         }
-        sync.nodes = nodes
-        docs = [doc for doc in sync.sync()]
+        sync.tree.__post_init__()
+        sync.root = sync.tree.build(nodes)
+        docs = [sort_list(doc) for doc in sync.sync()]
         assert docs[0]["_id"] == "abc"
         assert docs[0]["_source"] == {
             "_meta": {"rating": {"id": [1]}},
@@ -249,8 +252,9 @@ class TestParentSingleChildFkOnChild(object):
                 }
             ],
         }
-        sync.nodes = nodes
-        docs = [doc for doc in sync.sync()]
+        sync.tree.__post_init__()
+        sync.root = sync.tree.build(nodes)
+        docs = [sort_list(doc) for doc in sync.sync()]
         docs = sorted(docs, key=lambda k: k["_id"])
         assert docs[0]["_id"] == "abc"
         assert docs[0]["_source"] == {
@@ -296,8 +300,9 @@ class TestParentSingleChildFkOnChild(object):
                 }
             ],
         }
-        sync.nodes = nodes
-        docs = [doc for doc in sync.sync()]
+        sync.tree.__post_init__()
+        sync.root = sync.tree.build(nodes)
+        docs = [sort_list(doc) for doc in sync.sync()]
         assert docs[0]["_id"] == "abc"
         assert docs[0]["_source"] == {
             "_meta": {"rating": {"id": [1]}},
@@ -343,8 +348,9 @@ class TestParentSingleChildFkOnChild(object):
                 }
             ],
         }
-        sync.nodes = nodes
-        docs = [doc for doc in sync.sync()]
+        sync.tree.__post_init__()
+        sync.root = sync.tree.build(nodes)
+        docs = [sort_list(doc) for doc in sync.sync()]
         assert docs[0]["_id"] == "abc"
         assert docs[0]["_source"] == {
             "_meta": {"rating": {"id": [1]}},
@@ -377,7 +383,9 @@ class TestParentSingleChildFkOnChild(object):
             ],
         }
         sync.nodes = nodes
-        docs = [doc for doc in sync.sync()]
+        sync.tree.__post_init__()
+        sync.root = sync.tree.build(nodes)
+        docs = [sort_list(doc) for doc in sync.sync()]
         assert docs[0]["_id"] == "abc"
 
         assert docs[0]["_source"] == {
@@ -421,8 +429,9 @@ class TestParentSingleChildFkOnChild(object):
                 }
             ],
         }
-        sync.nodes = nodes
-        docs = [doc for doc in sync.sync()]
+        sync.tree.__post_init__()
+        sync.root = sync.tree.build(nodes)
+        docs = [sort_list(doc) for doc in sync.sync()]
 
         fields = ["_meta", "description", "isbn", "rating", "title"]
         assert sorted(docs[0]["_source"].keys()) == sorted(fields)
@@ -448,8 +457,9 @@ class TestParentSingleChildFkOnChild(object):
                 }
             ],
         }
-        sync.nodes = nodes
-        docs = [doc for doc in sync.sync()]
+        sync.tree.__post_init__()
+        sync.root = sync.tree.build(nodes)
+        docs = [sort_list(doc) for doc in sync.sync()]
         assert docs[2]["_source"] == {
             "_meta": {"rating": {"id": [3]}},
             "copyright": None,
@@ -460,6 +470,7 @@ class TestParentSingleChildFkOnChild(object):
             "buyer_id": None,
             "rating": {"book_isbn": "ghi", "id": 3, "value": 3.3},
             "title": "The Rabbit Club",
+            "tags": None,
         }
         assert_resync_empty(sync, nodes)
 
@@ -475,7 +486,7 @@ class TestParentSingleChildFkOnChild(object):
         }
         sync.es.close()
         with pytest.raises(RelationshipTypeError) as excinfo:
-            Tree(sync).build(nodes)
+            Tree(sync.models).build(nodes)
         assert 'Relationship type "qwerty" is invalid' in str(excinfo.value)
 
     def test_invalid_relationship_variant(self, sync):
@@ -493,7 +504,7 @@ class TestParentSingleChildFkOnChild(object):
         }
         sync.es.close()
         with pytest.raises(RelationshipVariantError) as excinfo:
-            Tree(sync).build(nodes)
+            Tree(sync.models).build(nodes)
         assert 'Relationship variant "abcdefg" is invalid' in str(
             excinfo.value
         )
@@ -510,7 +521,7 @@ class TestParentSingleChildFkOnChild(object):
         }
         sync.es.close()
         with pytest.raises(RelationshipAttributeError) as excinfo:
-            Tree(sync).build(nodes)
+            Tree(sync.models).build(nodes)
         assert f"Relationship attribute {set(['foo'])} is invalid" in str(
             excinfo.value
         )
@@ -529,8 +540,9 @@ class TestParentSingleChildFkOnChild(object):
                 }
             ],
         }
-        sync.nodes = nodes
-        docs = [doc for doc in sync.sync()]
+        sync.tree.__post_init__()
+        sync.root = sync.tree.build(nodes)
+        docs = [sort_list(doc) for doc in sync.sync()]
         sources = {doc["_id"]: doc["_source"] for doc in docs}
         assert sources["abc"]["_meta"] == {"rating": {"id": [1]}}
         assert sources["def"]["_meta"] == {"rating": {"id": [2]}}
@@ -551,9 +563,10 @@ class TestParentSingleChildFkOnChild(object):
                 }
             ],
         }
-        sync.nodes = nodes
+        sync.tree.__post_init__()
+        sync.root = sync.tree.build(nodes)
         with pytest.raises(ForeignKeyError) as excinfo:
-            [doc for doc in sync.sync()]
+            [sort_list(doc) for doc in sync.sync()]
         msg = (
             "No foreign key relationship between "
             '"public.book" and "public.city"'
@@ -563,9 +576,9 @@ class TestParentSingleChildFkOnChild(object):
     def test_missing_relationships(self, sync, data):
         """Relationships must be present between parent and child"""
         nodes = {"table": "book", "children": [{"table": "rating"}]}
-        sync.nodes = nodes
+        sync.tree.__post_init__()
         with pytest.raises(RelationshipError) as excinfo:
-            [doc for doc in sync.sync()]
+            sync.tree.build(nodes)
         assert 'Relationship not present on "public.rating"' in str(
             excinfo.value
         )
