@@ -16,7 +16,7 @@ from pgsync.node import Tree
 from pgsync.settings import NTHREADS_POLLDB
 from pgsync.sync import Sync
 
-from .helpers.utils import assert_resync_empty, noop, search
+from .helpers.utils import assert_resync_empty, noop, search, sort_list
 
 
 @pytest.mark.usefixtures("table_creator")
@@ -103,8 +103,9 @@ class TestParentSingleChildFkOnParent(object):
                 }
             ],
         }
-        sync.nodes = nodes
-        docs = [doc for doc in sync.sync()]
+        sync.tree.__post_init__()
+        sync.root = sync.tree.build(nodes)
+        docs = [sort_list(doc) for doc in sync.sync()]
         assert docs[0]["_id"] == "abc"
         assert docs[0]["_source"] == {
             "_meta": {"publisher": {"id": [1]}},
@@ -148,8 +149,9 @@ class TestParentSingleChildFkOnParent(object):
                 }
             ],
         }
-        sync.nodes = nodes
-        docs = [doc for doc in sync.sync()]
+        sync.tree.__post_init__()
+        sync.root = sync.tree.build(nodes)
+        docs = [sort_list(doc) for doc in sync.sync()]
         docs = sorted(docs, key=lambda k: k["_id"])
 
         assert docs[0]["_id"] == "abc"
@@ -195,8 +197,9 @@ class TestParentSingleChildFkOnParent(object):
                 }
             ],
         }
-        sync.nodes = nodes
-        docs = [doc for doc in sync.sync()]
+        sync.tree.__post_init__()
+        sync.root = sync.tree.build(nodes)
+        docs = [sort_list(doc) for doc in sync.sync()]
         assert docs[0]["_id"] == "abc"
         assert docs[0]["_source"] == {
             "_meta": {"publisher": {"id": [1]}},
@@ -240,8 +243,9 @@ class TestParentSingleChildFkOnParent(object):
                 }
             ],
         }
-        sync.nodes = nodes
-        docs = [doc for doc in sync.sync()]
+        sync.tree.__post_init__()
+        sync.root = sync.tree.build(nodes)
+        docs = [sort_list(doc) for doc in sync.sync()]
         docs = sorted(docs, key=lambda k: k["_id"])
         assert docs[0]["_id"] == "abc"
         assert docs[0]["_source"] == {
@@ -287,8 +291,9 @@ class TestParentSingleChildFkOnParent(object):
                 }
             ],
         }
-        sync.nodes = nodes
-        docs = [doc for doc in sync.sync()]
+        sync.tree.__post_init__()
+        sync.root = sync.tree.build(nodes)
+        docs = [sort_list(doc) for doc in sync.sync()]
         assert docs[0]["_id"] == "abc"
         assert docs[0]["_source"] == {
             "_meta": {"publisher": {"id": [1]}},
@@ -334,8 +339,9 @@ class TestParentSingleChildFkOnParent(object):
                 }
             ],
         }
-        sync.nodes = nodes
-        docs = [doc for doc in sync.sync()]
+        sync.tree.__post_init__()
+        sync.root = sync.tree.build(nodes)
+        docs = [sort_list(doc) for doc in sync.sync()]
         assert docs[0]["_id"] == "abc"
         assert docs[0]["_source"] == {
             "_meta": {"publisher": {"id": [1]}},
@@ -371,7 +377,9 @@ class TestParentSingleChildFkOnParent(object):
             ],
         }
         sync.nodes = nodes
-        docs = [doc for doc in sync.sync()]
+        sync.tree.__post_init__()
+        sync.root = sync.tree.build(nodes)
+        docs = [sort_list(doc) for doc in sync.sync()]
         assert docs[0]["_id"] == "abc"
         assert docs[0]["_source"] == {
             "_meta": {"publisher": {"id": [1]}},
@@ -424,9 +432,10 @@ class TestParentSingleChildFkOnParent(object):
                 }
             ],
         }
-        sync.nodes = nodes
-        docs = [doc for doc in sync.sync()]
 
+        sync.tree.__post_init__()
+        sync.root = sync.tree.build(nodes)
+        docs = [sort_list(doc) for doc in sync.sync()]
         fields = ["_meta", "description", "isbn", "publisher", "title"]
         assert sorted(docs[0]["_source"].keys()) == sorted(fields)
         assert sorted(docs[1]["_source"].keys()) == sorted(fields)
@@ -451,8 +460,9 @@ class TestParentSingleChildFkOnParent(object):
                 }
             ],
         }
-        sync.nodes = nodes
-        docs = [doc for doc in sync.sync()]
+        sync.tree.__post_init__()
+        sync.root = sync.tree.build(nodes)
+        docs = [sort_list(doc) for doc in sync.sync()]
         assert docs[2]["_source"] == {
             "_meta": {"publisher": {"id": [3]}},
             "copyright": None,
@@ -463,6 +473,7 @@ class TestParentSingleChildFkOnParent(object):
             "seller_id": None,
             "buyer_id": None,
             "title": "The Rabbit Club",
+            "tags": None,
         }
         assert_resync_empty(sync, nodes)
 
@@ -477,8 +488,9 @@ class TestParentSingleChildFkOnParent(object):
             ],
         }
         sync.es.close()
+        sync.tree.__post_init__()
         with pytest.raises(RelationshipTypeError) as excinfo:
-            Tree(sync).build(nodes)
+            Tree(sync.models).build(nodes)
         assert 'Relationship type "qwerty" is invalid' in str(excinfo.value)
 
     def test_invalid_relationship_variant(self, sync):
@@ -495,8 +507,9 @@ class TestParentSingleChildFkOnParent(object):
             ],
         }
         sync.es.close()
+        sync.tree.__post_init__()
         with pytest.raises(RelationshipVariantError) as excinfo:
-            Tree(sync).build(nodes)
+            Tree(sync.models).build(nodes)
         assert 'Relationship variant "abcdefg" is invalid' in str(
             excinfo.value
         )
@@ -512,8 +525,9 @@ class TestParentSingleChildFkOnParent(object):
             ],
         }
         sync.es.close()
+        sync.tree.__post_init__()
         with pytest.raises(RelationshipAttributeError) as excinfo:
-            Tree(sync).build(nodes)
+            Tree(sync.models).build(nodes)
         assert f"Relationship attribute {set(['foo'])} is invalid" in str(
             excinfo.value
         )
@@ -532,8 +546,9 @@ class TestParentSingleChildFkOnParent(object):
                 }
             ],
         }
-        sync.nodes = nodes
-        docs = [doc for doc in sync.sync()]
+        sync.tree.__post_init__()
+        sync.root = sync.tree.build(nodes)
+        docs = [sort_list(doc) for doc in sync.sync()]
         sources = {doc["_id"]: doc["_source"] for doc in docs}
         assert sources["abc"]["_meta"] == {"publisher": {"id": [1]}}
         assert sources["def"]["_meta"] == {"publisher": {"id": [2]}}
@@ -554,9 +569,10 @@ class TestParentSingleChildFkOnParent(object):
                 }
             ],
         }
-        sync.nodes = nodes
+        sync.tree.__post_init__()
+        sync.root = sync.tree.build(nodes)
         with pytest.raises(ForeignKeyError) as excinfo:
-            [doc for doc in sync.sync()]
+            [sort_list(doc) for doc in sync.sync()]
         msg = (
             "No foreign key relationship between "
             '"public.book" and "public.city"'
@@ -566,9 +582,9 @@ class TestParentSingleChildFkOnParent(object):
     def test_missing_relationships(self, sync, data):
         """Relationships must be present between parent and child"""
         nodes = {"table": "book", "children": [{"table": "publisher"}]}
-        sync.nodes = nodes
+        sync.tree.__post_init__()
         with pytest.raises(RelationshipError) as excinfo:
-            [doc for doc in sync.sync()]
+            sync.tree.build(nodes)
         assert 'Relationship not present on "public.publisher"' in str(
             excinfo.value
         )
